
(define-syntax scons
  (syntax-rules () ((scons head tail) (delay (cons head tail)))))

(define snil (delay '()))
(define (scar stream) (car (force stream)))
(define (scdr stream) (cdr (force stream)))
(define (snull? stream) (null? (force stream)))

(define (stream->list s)
  (if (snull? s) '()
    (cons (scar s) (stream->list (scdr s)))))

(define (list->stream ls)
  (if (null? ls) snil
    (scons (car ls) (list->stream (cdr ls)))))

(define (string->stream str)
  (let stream-from ((index 0))
    (if (>= index (string-length str)) snil
      (scons (string-ref str index) (stream-from (+ index 1))))))

(define (port->stream port)
  (delay
    (let ((ch (read-char port)))
      (if (eof-object? ch) '()
	(cons ch (port->stream port))))))

(define (stream-take s n)
  (if (or (snull? s) (= n 0)) snil
    (scons (scar s) (stream-take (scdr s) (- n 1)))))

(define (stream-drop s n)
  (if (or (snull? s) (= n 0)) s
    (stream-drop (scdr s) (- n 1))))

(define (stream-get-line lineno stream)
  (if (= lineno 1) (list->string (stream-first-line stream))
    (stream-get-line (- lineno 1) (stream-drop-line stream))))
(define (stream-first-line stream)
  (if (or (snull? stream) (eqv? (scar stream) #\newline)) '()
    (cons (scar stream) (stream-first-line (scdr stream)))))
(define (stream-drop-line stream)
  (cond ((snull? stream) stream)
	((eqv? (scar stream) #\newline) (scdr stream))
	(else (stream-drop-line (scdr stream)))))

